@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2021 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
@@ -37,23 +37,22 @@
 **
 ****************************************************************************/
 
+#include "androiddeadlockprotector.h"
 #include "androidjniaccessibility.h"
 #include "androidjnimain.h"
 #include "qandroidplatformintegration.h"
 #include "qpa/qplatformaccessibility.h"
-#include <QtAccessibilitySupport/private/qaccessiblebridgeutils_p.h>
+#include <QtGui/private/qaccessiblebridgeutils_p.h>
 #include "qguiapplication.h"
 #include "qwindow.h"
 #include "qrect.h"
 #include "QtGui/qaccessible.h"
 #include <QtCore/qmath.h>
 #include <QtCore/private/qjnihelpers_p.h>
-#include <QtCore/private/qjni_p.h>
+#include <QtCore/QJniObject>
 #include <QtGui/private/qhighdpiscaling_p.h>
 #include <QtCore/QObject>
 
-#include "qdebug.h"
-
 static const char m_qtTag[] = "Qt A11Y";
 static const char m_classErrorMsg[] = "Can't find class \"%s\"";
 
@@ -70,6 +69,7 @@ namespace QtAndroidAccessibility
     static jmethodID m_setEnabledMethodID = 0;
     static jmethodID m_setFocusableMethodID = 0;
     static jmethodID m_setFocusedMethodID = 0;
+    static jmethodID m_setHeadingMethodID = 0;
     static jmethodID m_setScrollableMethodID = 0;
     static jmethodID m_setTextSelectionMethodID = 0;
     static jmethodID m_setVisibleToUserMethodID = 0;
@@ -81,7 +81,7 @@ namespace QtAndroidAccessibility
     // Because of that almost every method here is split into two parts.
     // The _helper part is executed in the context of m_accessibilityContext
     // on the main thread. The other part is executed in Java thread.
-    static QObject *m_accessibilityContext = nullptr;
+    static QPointer<QObject> m_accessibilityContext = nullptr;
 
     // This method is called from the Qt main thread, and normally a
     // QGuiApplication instance will be used as a parent.
@@ -95,13 +95,27 @@ namespace QtAndroidAccessibility
     template <typename Func, typename Ret>
     void runInObjectContext(QObject *context, Func &&func, Ret *retVal)
     {
-        QMetaObject::invokeMethod(context, func, Qt::BlockingQueuedConnection, retVal);
+        AndroidDeadlockProtector protector;
+        if (!protector.acquire()) {
+            __android_log_print(ANDROID_LOG_WARN, m_qtTag,
+                                "Could not run accessibility call in object context, accessing "
+                                "main thread could lead to deadlock");
+            return;
+        }
+
+        if (!QtAndroid::blockEventLoopsWhenSuspended()
+            || QGuiApplication::applicationState() != Qt::ApplicationSuspended) {
+            QMetaObject::invokeMethod(context, func, Qt::BlockingQueuedConnection, retVal);
+        } else {
+            __android_log_print(ANDROID_LOG_WARN, m_qtTag,
+                                "Could not run accessibility call in object context, event loop suspended.");
+        }
     }
 
     void initialize()
     {
-        QJNIObjectPrivate::callStaticMethod<void>(QtAndroid::applicationClass(),
-                                                  "initializeAccessibility");
+        QJniObject::callStaticMethod<void>(QtAndroid::applicationClass(),
+                                           "initializeAccessibility");
     }
 
     bool isActive()
@@ -122,7 +136,7 @@ namespace QtAndroidAccessibility
 
     QAccessibleInterface *interfaceFromId(jint objectId)
     {
-        QAccessibleInterface *iface = 0;
+        QAccessibleInterface *iface = nullptr;
         if (objectId == -1) {
             QWindow *win = qApp->focusWindow();
             if (win)
@@ -133,9 +147,9 @@ namespace QtAndroidAccessibility
         return iface;
     }
 
-    void notifyLocationChange()
+    void notifyLocationChange(uint accessibilityObjectId)
     {
-        QtAndroid::notifyAccessibilityLocationChange();
+        QtAndroid::notifyAccessibilityLocationChange(accessibilityObjectId);
     }
 
     static int parentId_helper(int objectId); // forward declaration
@@ -159,6 +173,11 @@ namespace QtAndroidAccessibility
         QtAndroid::notifyValueChanged(accessibilityObjectId, value);
     }
 
+    void notifyScrolledEvent(uint accessiblityObjectId)
+    {
+        QtAndroid::notifyScrolledEvent(accessiblityObjectId);
+    }
+
     static QVarLengthArray<int, 8> childIdListForAccessibleObject_helper(int objectId)
     {
         QAccessibleInterface *iface = interfaceFromId(objectId);
@@ -216,7 +235,7 @@ namespace QtAndroidAccessibility
         return result;
     }
 
-    static QRect screenRect_helper(int objectId)
+    static QRect screenRect_helper(int objectId, bool clip = true)
     {
         QRect rect;
         QAccessibleInterface *iface = interfaceFromId(objectId);
@@ -224,7 +243,7 @@ namespace QtAndroidAccessibility
             rect = QHighDpi::toNativePixels(iface->rect(), iface->window());
         }
         // If the widget is not fully in-bound in its parent then we have to clip the rectangle to draw
-        if (iface && iface->parent() && iface->parent()->isValid()) {
+        if (clip && iface && iface->parent() && iface->parent()->isValid()) {
             const auto parentRect = QHighDpi::toNativePixels(iface->parent()->rect(), iface->parent()->window());
             rect = rect.intersected(parentRect);
         }
@@ -252,7 +271,7 @@ namespace QtAndroidAccessibility
             QPoint pos = QHighDpi::fromNativePixels(QPoint(int(x), int(y)), root->window());
 
             QAccessibleInterface *child = root->childAt(pos.x(), pos.y());
-            QAccessibleInterface *lastChild = 0;
+            QAccessibleInterface *lastChild = nullptr;
             while (child && (child != lastChild)) {
                 lastChild = child;
                 child = child->childAt(pos.x(), pos.y());
@@ -326,23 +345,43 @@ namespace QtAndroidAccessibility
     static jboolean scrollForward(JNIEnv */*env*/, jobject /*thiz*/, jint objectId)
     {
         bool result = false;
+
+        const auto& ids = childIdListForAccessibleObject_helper(objectId);
+        if (ids.isEmpty())
+            return false;
+
+        const int firstChildId = ids.first();
+        const QRect oldPosition = screenRect_helper(firstChildId, false);
+
         if (m_accessibilityContext) {
             runInObjectContext(m_accessibilityContext, [objectId]() {
                 return scroll_helper(objectId, QAccessibleActionInterface::increaseAction());
             }, &result);
         }
-        return result;
+
+        // Don't check for position change if the call was not successful
+        return result && oldPosition != screenRect_helper(firstChildId, false);
     }
 
     static jboolean scrollBackward(JNIEnv */*env*/, jobject /*thiz*/, jint objectId)
     {
         bool result = false;
+
+        const auto& ids = childIdListForAccessibleObject_helper(objectId);
+        if (ids.isEmpty())
+            return false;
+
+        const int firstChildId = ids.first();
+        const QRect oldPosition = screenRect_helper(firstChildId, false);
+
         if (m_accessibilityContext) {
             runInObjectContext(m_accessibilityContext, [objectId]() {
                 return scroll_helper(objectId, QAccessibleActionInterface::decreaseAction());
             }, &result);
         }
-        return result;
+
+        // Don't check for position change if the call was not successful
+        return result && oldPosition != screenRect_helper(firstChildId, false);
     }
 
 
@@ -361,7 +400,7 @@ if (!clazz) { \
         QAccessibleValueInterface *valueIface = iface->valueInterface();
         if (valueIface) {
             const QVariant valueVar = valueIface->currentValue();
-            const auto type = static_cast<QMetaType::Type>(valueVar.type());
+            const auto type = valueVar.typeId();
             if (type == QMetaType::Double || type == QMetaType::Float) {
                 // QVariant's toString() formats floating-point values with
                 // FloatingPointShortest, which is not an accessible
@@ -374,7 +413,7 @@ if (!clazz) { \
                 const double step = qAbs(valueIface->minimumStepSize().toDouble(&stepIsValid));
                 if (!stepIsValid || qFuzzyIsNull(step)) {
                     // Ignore step, use default precision
-                    valueStr = qFuzzyIsNull(val) ? QStringLiteral("0") : QString::number(val, 'f');
+                    valueStr = qFuzzyIsNull(val) ? u"0"_qs : QString::number(val, 'f');
                 } else {
                     const int precision = [](double s) {
                         int count = 0;
@@ -398,7 +437,7 @@ if (!clazz) { \
                         }
                         return count;
                     }(step);
-                    valueStr = qFuzzyIsNull(val / step) ? QStringLiteral("0")
+                    valueStr = qFuzzyIsNull(val / step) ? u"0"_qs
                                                         : QString::number(val, 'f', precision);
                 }
             } else {
@@ -412,12 +451,10 @@ if (!clazz) { \
     {
         QAccessibleInterface *iface = interfaceFromId(objectId);
         const QString value = textFromValue(iface);
-        QJNIEnvironmentPrivate env;
+        QJniEnvironment env;
         jstring jstr = env->NewString((jchar*)value.constData(), (jsize)value.size());
-#ifdef QT_DEBUG
-        env->ExceptionDescribe();
-#endif // QT_DEBUG
-        env->ExceptionClear();
+        if (env.checkAndClearExceptions())
+            __android_log_print(ANDROID_LOG_WARN, m_qtTag, "Failed to create jstring");
         return jstr;
     }
 
@@ -433,10 +470,13 @@ if (!clazz) { \
                 desc = iface->text(QAccessible::Value);
                 hasValue = !desc.isEmpty();
             }
-            if (!hasValue) {
-                if (!desc.isEmpty())
-                    desc.append(QChar(QChar::Space));
-                desc.append(textFromValue(iface));
+            if (!hasValue && iface->valueInterface()) {
+                const QString valueStr = textFromValue(iface);
+                if (!valueStr.isEmpty()) {
+                    if (!desc.isEmpty())
+                        desc.append(QChar(QChar::Space));
+                    desc.append(valueStr);
+                }
             }
         }
         return desc;
@@ -464,6 +504,7 @@ if (!clazz) { \
     {
         bool valid = false;
         QAccessible::State state;
+        QAccessible::Role role;
         QStringList actions;
         QString description;
         bool hasTextSelection = false;
@@ -478,6 +519,7 @@ if (!clazz) { \
         if (iface && iface->isValid()) {
             info.valid = true;
             info.state = iface->state();
+            info.role = iface->role();
             info.actions = QAccessibleBridgeUtils::effectiveActionNames(iface);
             info.description = descriptionForInterface(iface);
             QAccessibleTextInterface *textIface = iface->textInterface();
@@ -521,9 +563,11 @@ if (!clazz) { \
         env->CallVoidMethod(node, m_setEnabledMethodID, !info.state.disabled);
         env->CallVoidMethod(node, m_setFocusableMethodID, (bool)info.state.focusable);
         env->CallVoidMethod(node, m_setFocusedMethodID, (bool)info.state.focused);
+        if (m_setHeadingMethodID)
+            env->CallVoidMethod(node, m_setHeadingMethodID, info.role == QAccessible::Heading);
         env->CallVoidMethod(node, m_setVisibleToUserMethodID, !info.state.invisible);
         env->CallVoidMethod(node, m_setScrollableMethodID, hasIncreaseAction || hasDecreaseAction);
-        env->CallVoidMethod(node, m_setClickableMethodID, hasClickableAction);
+        env->CallVoidMethod(node, m_setClickableMethodID, hasClickableAction || info.role == QAccessible::Link);
 
         // Add ACTION_CLICK
         if (hasClickableAction)
@@ -569,7 +613,7 @@ if (!clazz) { \
     bool registerNatives(JNIEnv *env)
     {
         jclass clazz;
-        FIND_AND_CHECK_CLASS("org/qtproject/qt5/android/accessibility/QtNativeAccessibility");
+        FIND_AND_CHECK_CLASS("org/qtproject/qt/android/accessibility/QtNativeAccessibility");
         jclass appClass = static_cast<jclass>(env->NewGlobalRef(clazz));
 
         if (env->RegisterNatives(appClass, methods, sizeof(methods) / sizeof(methods[0])) < 0) {
@@ -587,6 +631,9 @@ if (!clazz) { \
         GET_AND_CHECK_STATIC_METHOD(m_setEnabledMethodID, nodeInfoClass, "setEnabled", "(Z)V");
         GET_AND_CHECK_STATIC_METHOD(m_setFocusableMethodID, nodeInfoClass, "setFocusable", "(Z)V");
         GET_AND_CHECK_STATIC_METHOD(m_setFocusedMethodID, nodeInfoClass, "setFocused", "(Z)V");
+        if (QtAndroidPrivate::androidSdkVersion() >= 28) {
+            GET_AND_CHECK_STATIC_METHOD(m_setHeadingMethodID, nodeInfoClass, "setHeading", "(Z)V");
+        }
         GET_AND_CHECK_STATIC_METHOD(m_setScrollableMethodID, nodeInfoClass, "setScrollable", "(Z)V");
         GET_AND_CHECK_STATIC_METHOD(m_setVisibleToUserMethodID, nodeInfoClass, "setVisibleToUser", "(Z)V");
         GET_AND_CHECK_STATIC_METHOD(m_setTextSelectionMethodID, nodeInfoClass, "setTextSelection", "(II)V");
