@@ -1162,13 +1162,17 @@ void tst_QDockWidget::createTestWidgets(QMainWindow* &mainWindow, QPointer<QWidg
     mainWindow->setDockOptions(QMainWindow::AllowTabbedDocks | QMainWindow::GroupedDragging);
     mainWindow->move(m_topLeft);
 
+    const int minWidth = QApplication::style()->pixelMetric(QStyle::PM_TitleBarHeight);
+    const QSize minSize(minWidth, 2 * minWidth);
     d1 = new QDockWidget(mainWindow);
+    d1->setMinimumSize(minSize);
     d1->setWindowTitle("I am D1");
     d1->setObjectName("D1");
     d1->setFeatures(QDockWidget::DockWidgetFeatureMask);
     d1->setAllowedAreas(Qt::DockWidgetArea::AllDockWidgetAreas);
 
     d2 = new QDockWidget(mainWindow);
+    d2->setMinimumSize(minSize);
     d2->setWindowTitle("I am D2");
     d2->setObjectName("D2");
     d2->setFeatures(QDockWidget::DockWidgetFeatureMask);
@@ -1381,8 +1385,6 @@ void tst_QDockWidget::floatingTabs()
 
     // Both dock widgets must no longer be floating
     // disabled due to flakiness on macOS and Windows
-    //QTRY_VERIFY(!d1->isFloating());
-    //QTRY_VERIFY(!d2->isFloating());
     if (d1->isFloating())
         qWarning("OS flakiness: D1 is docked and reports being floating");
     if (d2->isFloating())
@@ -1390,7 +1392,18 @@ void tst_QDockWidget::floatingTabs()
 
     // Now MainWindow has to have a floatingTab child
     QPointer<QDockWidgetGroupWindow> ftabs;
-    QTRY_VERIFY(checkFloatingTabs(mainWindow, ftabs, QList<QDockWidget*>() << d1 << d2));
+    QTRY_VERIFY(checkFloatingTabs(mainWindow, ftabs, QList<QDockWidget *>() << d1 << d2));
+
+    // Hide both dock widgets. Verify that the group window is also hidden.
+    qCDebug(lcTestDockWidget) << "*** Hide and show tabbed dock widgets ***";
+    d1->hide();
+    d2->hide();
+    QTRY_VERIFY(ftabs->isHidden());
+
+    // Show both dockwidgets again. Verify that the group window is visible.
+    d1->show();
+    d2->show();
+    QTRY_VERIFY(ftabs->isVisible());
 
     /*
      * replug both dock widgets into their initial position
@@ -1438,10 +1451,10 @@ void tst_QDockWidget::floatingTabs()
     qCDebug(lcTestDockWidget) << "Checking path consistency" << layout->layoutState.indexOf(d1) << layout->layoutState.indexOf(d2);
 
     // Path1 must be identical
-    QTRY_VERIFY(path1 == layout->layoutState.indexOf(d1));
+    QTRY_COMPARE(path1, layout->layoutState.indexOf(d1));
 
     // d1 must have a gap item due to size change
-    QTRY_VERIFY(layout->layoutState.indexOf(d2) == QList<int>() << path2 << 0);
+    QTRY_COMPARE(layout->layoutState.indexOf(d2), QList<int>() << path2 << 0);
 #else
     QSKIP("test requires -developer-build option");
 #endif // QT_BUILD_INTERNAL
